/*
 * This file is part of the OpenNMS(R) Application.
 *
 * OpenNMS(R) is Copyright (C) 2006 The OpenNMS Group, Inc.  All rights reserved.
 * OpenNMS(R) is a derivative work, containing both original code, included code and modified
 * code that was published under the GNU General Public License. Copyrights for modified
 * and included code are below.
 *
 * OpenNMS(R) is a registered trademark of The OpenNMS Group, Inc.
 *
 * Modifications:
 *
 * 2007 Apr 05: Created this file. - dj@opennms.org
 *
 * Copyright (C) 2007 The OpenNMS Group, Inc.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * For more information contact:
 *      OpenNMS Licensing       <license@opennms.org>
 *      http://www.opennms.org/
 *      http://www.opennms.com/
 */
package org.opennms.netmgt.dao.support;

import java.util.Collections;
import java.util.HashSet;

import org.opennms.netmgt.model.AttributeVisitor;
import org.opennms.netmgt.model.OnmsAttribute;
import org.opennms.netmgt.model.OnmsResource;
import org.opennms.netmgt.model.RrdGraphAttribute;
import org.opennms.test.ThrowableAnticipator;
import org.opennms.test.mock.EasyMockUtils;

import junit.framework.TestCase;

/**
 * @author <a href="mailto:dj@opennms.org">DJ Gregor</a>
 */
public class AttributeMatchingResourceVisitorTest extends TestCase {
    private EasyMockUtils m_mocks = new EasyMockUtils();
    private AttributeVisitor m_attributeVisitor = m_mocks.createMock(AttributeVisitor.class);
    
    public void testAfterPropertiesSet() throws Exception {
        AttributeMatchingResourceVisitor resourceVisitor = new AttributeMatchingResourceVisitor();
        resourceVisitor.setAttributeVisitor(m_attributeVisitor);
        resourceVisitor.setAttributeMatch("ifInOctets");
        resourceVisitor.afterPropertiesSet();
    }


    public void testAfterPropertiesSetNoAttributeVisitor() throws Exception {
        AttributeMatchingResourceVisitor resourceVisitor = new AttributeMatchingResourceVisitor();
        
        ThrowableAnticipator ta = new ThrowableAnticipator();
        ta.anticipate(new IllegalStateException("property attributeVisitor must be set to a non-null value"));
        
        resourceVisitor.setAttributeVisitor(null);
        resourceVisitor.setAttributeMatch("ifInOctets");

        try {
            resourceVisitor.afterPropertiesSet();
        } catch (Throwable t) {
            ta.throwableReceived(t);
        }
        ta.verifyAnticipated();
    }
    
    public void testAfterPropertiesSetNoResourceTypeMatch() throws Exception {
        AttributeMatchingResourceVisitor resourceVisitor = new AttributeMatchingResourceVisitor();
        
        ThrowableAnticipator ta = new ThrowableAnticipator();
        ta.anticipate(new IllegalStateException("property attributeMatch must be set to a non-null value"));

        resourceVisitor.setAttributeVisitor(m_attributeVisitor);
        resourceVisitor.setAttributeMatch(null);

        try {
            resourceVisitor.afterPropertiesSet();
        } catch (Throwable t) {
            ta.throwableReceived(t);
        }
        ta.verifyAnticipated();
    }

    public void testVisitWithMatch() throws Exception {
        AttributeMatchingResourceVisitor resourceVisitor = new AttributeMatchingResourceVisitor();
        resourceVisitor.setAttributeVisitor(m_attributeVisitor);
        resourceVisitor.setAttributeMatch("ifInOctets");
        resourceVisitor.afterPropertiesSet();

        MockResourceType resourceType = new MockResourceType();
        resourceType.setName("interfaceSnmp");
        OnmsAttribute attribute = new RrdGraphAttribute("ifInOctets", "something", "something else");
        OnmsResource resource = new OnmsResource("1", "Node One", resourceType, Collections.singleton(attribute));
        m_attributeVisitor.visit(attribute);

        m_mocks.replayAll();
        resourceVisitor.visit(resource);
        m_mocks.verifyAll();
    }
    
    public void testVisitWithoutMatch() throws Exception {
        AttributeMatchingResourceVisitor resourceVisitor = new AttributeMatchingResourceVisitor();
        resourceVisitor.setAttributeVisitor(m_attributeVisitor);
        resourceVisitor.setAttributeMatch("ifInOctets");
        resourceVisitor.afterPropertiesSet();

        MockResourceType resourceType = new MockResourceType();
        resourceType.setName("something other than interfaceSnmp");
        OnmsResource resource = new OnmsResource("1", "Node One", resourceType, new HashSet<OnmsAttribute>(0));

        m_mocks.replayAll();
        resourceVisitor.visit(resource);
        m_mocks.verifyAll();
    }
}
